<?php

namespace App\Form\Type;

use App\Entity\Import;
use App\Entity\User;
use Doctrine\ORM\EntityManagerInterface;
use Sonata\MediaBundle\Form\Type\MediaType;
use Symfony\Component\Form\AbstractType;
use Symfony\Component\Form\Extension\Core\Type\HiddenType;
use Symfony\Component\Form\Extension\Core\Type\SubmitType;
use Symfony\Component\Form\FormBuilderInterface;
use Symfony\Component\OptionsResolver\OptionsResolver;
use Symfony\Component\Security\Core\Security;

class ImportFormType extends AbstractType
{
    protected $em;
    protected $security;

    public function __construct(EntityManagerInterface $em, Security $security)
    {
        $this->em = $em;
        $this->security = $security;
    }

    public function buildForm(FormBuilderInterface $builder, array $options)
    {
        if (empty($this->security) && !empty($this->security->getUser())) {
            throw new \Exception("Opération impossible ! Utilisateur non connecté !");
        }
        $builder
            ->add('media', MediaType::class, array(
                'provider' => 'sonata.media.provider.file',
                'context' => 'import',
                'label' => 'Fichier .csv',
                // 'show_unlink' => false
            ))

            ->add('user', HiddenType::class, array(
                'data_class' => null,
                'data' => $this->security->getUser()->getId(),
                'entity_class' => User::class,
                'em' => $this->em
            ))
            ->add('save', SubmitType::class, ['label' => "Importer les données"])
        ;
    }

    public function configureOptions(OptionsResolver $resolver)
    {
        $resolver->setDefaults([
            'data_class' => Import::class,
            'cascade_validation' => true
        ]);
    }

    public function getBlockPrefix()
    {
        return 'formImport';
    }
}
